#property link          "https://www.earnforex.com/metatrader-expert-advisors/hidden-stop-loss-take-profit-ghost-sl-tp/"
#property version       "1.02"
#property strict
#property copyright     "EarnForex.com - 2019-2021"
#property description   "This Expert Advisor will Execute Stop Loss and Take Profit" 
#property description   "without letting your broker know where they are."
#property description   " "
#property description   "WARNING : You use this software at your own risk."
#property description   "The creator of these plugins cannot be held responsible for any damage or loss."
#property description   " "
#property description   "Find More on EarnForex.com"
#property icon          "\\Files\\EF-Icon-64x64px.ico"

#include <MQLTA ErrorHandling.mqh>
#include <MQLTA Utils.mqh>

enum ENUM_CONSIDER{
   All=-1,        //ALL ORDERS
   Buy=OP_BUY,    //BUY ONLY
   Sell=OP_SELL,  //SELL ONLY
};

enum ENUM_CUSTOMTIMEFRAMES{
   CURRENT=PERIOD_CURRENT,             //CURRENT PERIOD
   M1=PERIOD_M1,                       //M1
   M5=PERIOD_M5,                       //M5
   M15=PERIOD_M15,                     //M15
   M30=PERIOD_M30,                     //M30
   H1=PERIOD_H1,                       //H1
   H4=PERIOD_H4,                       //H4
   D1=PERIOD_D1,                       //D1
   W1=PERIOD_W1,                       //W1
   MN1=PERIOD_MN1,                     //MN1
};

enum ENUM_REFERENCE_PRICE{
   PRICE_BID=1,      //BID PRICE
   PRICE_ASK=2,      //ASK PRICE
};


input string Comment_1="====================";        //Expert Advisor Settings
input ENUM_REFERENCE_PRICE BuyRefPrice=PRICE_BID;     //Reference Price For Buy Orders
input ENUM_REFERENCE_PRICE SellRefPrice=PRICE_ASK;    //Reference Price For Sell Orders
input int SLDistance=500;                             //Stop Loss In Points
input int TPDistance=500;                             //Take Profit In Points
input int Slippage=10;                                //Slippage In Points
input string Comment_2="====================";        //Orders Filtering Options
bool OnlyCurrentSymbol=true;                          //Apply To Current Symbol Only
input ENUM_CONSIDER OnlyType=All;                     //Apply To
input bool UseMagic=false;                            //Filter By Magic Number
input int MagicNumber=0;                              //Magic Number (if above is true)
input bool UseComment=false;                          //Filter By Comment
input string CommentFilter="";                        //Comment (if above is true)
input bool EnableMonitoringParam=false;               //Enable Monitoring
input string Comment_3="====================";        //Notification Options
input bool EnableNotify=false;                        //Enable Notifications feature
input bool SendAlert=true;                            //Send Alert Notification
input bool SendApp=true;                              //Send Notification to Mobile
input bool SendEmail=true;                            //Send Notification via Email
input string Comment_3a="====================";       //Graphical Window
input bool ShowPanel=true;                            //Show Graphical Panel
input string IndicatorName="MQLTA-GTPSL";             //Indicator Name (to name the objects)
input int Xoff=20;                                    //Horizontal spacing for the control panel
input int Yoff=20;                                    //Vertical spacing for the control panel

int OrderOpRetry=5;

bool EnableMonitoring=EnableMonitoringParam;


int OnInit(){
   CleanPanel();
   
   EnableMonitoring=EnableMonitoringParam;
   if(ShowPanel) DrawPanel();
   return(INIT_SUCCEEDED);
   
   
}

void OnDeinit(const int reason){
   CleanPanel();
   
}

void OnTick(){
   CleanPanel();

   if(EnableMonitoring) MonitoringTPSL();
   if(ShowPanel) DrawPanel();
   
}


void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam){

   if(id==CHARTEVENT_OBJECT_CLICK){
      if(sparam==PanelEnableDisable){
         ChangeMonitoringEnabled();
      }
   }
   if(id==CHARTEVENT_KEYDOWN){
      if(lparam==27){
         if(MessageBox("Are you sure you want to close the EA?","EXIT ?",MB_YESNO)==IDYES){
            ExpertRemove();
         }
      }
   }
   
}


void MonitoringTPSL(){
   for(int i=0;i<OrdersTotal();i++) {
      if(OrderSelect( i, SELECT_BY_POS, MODE_TRADES ) == false ) {
         int Error=GetLastError();
         string ErrorText=GetLastErrorText(Error);
         Print("ERROR - Unable to select the order - ",Error);
         Print("ERROR - ",ErrorText);
         break;
      }
      if(OnlyCurrentSymbol && OrderSymbol()!=Symbol()) continue;
      if(UseMagic && OrderMagicNumber()!=MagicNumber) continue;
      if(UseComment && StringFind(OrderComment(),CommentFilter)<0) continue;
      if(OnlyType!=All &&OrderType()!=OnlyType) continue;
      
      string Instrument=OrderSymbol();
      double ReferencePrice=0;
      if(OrderType()==OP_BUY){
         RefreshRates();
         if(BuyRefPrice==PRICE_BID) ReferencePrice=Bid;
         if(BuyRefPrice==PRICE_ASK) ReferencePrice=Ask;
         if(OrderOpenPrice()+TPDistance*Point<ReferencePrice || OrderOpenPrice()-SLDistance*Point>ReferencePrice) CloseOrder(OrderTicket(),OrderLots(),ReferencePrice);
      }
      if(OrderType()==OP_SELL){
         RefreshRates();
         if(SellRefPrice==PRICE_ASK) ReferencePrice=Ask;
         if(SellRefPrice==PRICE_BID) ReferencePrice=Bid;
         if(OrderOpenPrice()-TPDistance*Point>ReferencePrice || OrderOpenPrice()+SLDistance*Point<ReferencePrice) CloseOrder(OrderTicket(),OrderLots(),ReferencePrice);
      }
   }
}


//Close Single Order Function adjusted to handle errors and retry multiple times
void CloseOrder(int Ticket, double Lots, double CurrentPrice){
   //Try to close the order by ticket number multiple times in case of failure
   for(int i=1; i<=OrderOpRetry; i++){
      //Send the close command
      bool res=OrderClose(Ticket,Lots,CurrentPrice,Slippage,Red);
      //If the close was successful print the resul and exit the function
      if(res){
         Print("TRADE - CLOSE SUCCESS - Order ",Ticket," closed at price ",CurrentPrice);
         NotifyStopLossUpdate(Ticket);
         break;
      }
      //If the close failed print the error
      else{
         int Error=GetLastError();
         string ErrorText=GetLastErrorText(Error);
         Print("ERROR - CLOSE FAILED - error closing order ",Ticket," return error: ",Error," - ",ErrorText);
      } 
   }
   return;
}


void NotifyStopLossUpdate(int OrderNumber){
   if(!EnableNotify) return;
   if(!SendAlert && !SendApp && !SendEmail) return;
   string EmailSubject=IndicatorName+" "+Symbol()+" Notification ";
   string EmailBody="\r\n"+AccountCompany()+" - "+AccountName()+" - "+IntegerToString(AccountNumber())+"\r\n\r\n"+IndicatorName+" Notification for "+Symbol()+"\r\n\r\n";
   EmailBody+="Order "+string(OrderNumber)+" was closed\r\n\r\n";
   string AlertText=IndicatorName+" - "+Symbol()+" Notification\r\n";
   AlertText+="Order "+string(OrderNumber)+" was closed";
   string AppText=AccountCompany()+" - "+AccountName()+" - "+IntegerToString(AccountNumber())+" - "+IndicatorName+" - "+Symbol()+" - ";
   AppText+="Order "+string(OrderNumber)+" was closed";
   if(SendAlert) Alert(AlertText);
   if(SendEmail){
      if(!SendMail(EmailSubject,EmailBody)) Print("Error sending email "+IntegerToString(GetLastError()));
   }
   if(SendApp){
      if(!SendNotification(AppText)) Print("Error sending notification "+IntegerToString(GetLastError()));
   }
   datetime LastNotification=TimeCurrent();
   Print(IndicatorName+"-"+Symbol()+" last notification sent "+TimeToString(LastNotification));
}


string PanelBase=IndicatorName+"-P-BAS";
string PanelLabel=IndicatorName+"-P-LAB";
string PanelEnableDisable=IndicatorName+"-P-ENADIS";

int PanelMovX=50;
int PanelMovY=20;
int PanelLabX=150;
int PanelLabY=PanelMovY;
int PanelRecX=PanelLabX+4;

void DrawPanel(){
   string PanelText="MQLTA GTPSL";
   string PanelToolTip="Ghost Take Profit And Stop Loss By MQLTA";
   CleanPanel();
   int Rows=1;
   ObjectCreate(0,PanelBase,OBJ_RECTANGLE_LABEL,0,0,0);
   ObjectSet(PanelBase,OBJPROP_XDISTANCE,Xoff);
   ObjectSet(PanelBase,OBJPROP_YDISTANCE,Yoff);
   ObjectSetInteger(0,PanelBase,OBJPROP_XSIZE,PanelRecX);
   ObjectSetInteger(0,PanelBase,OBJPROP_YSIZE,(PanelMovY+2)*1+2);
   ObjectSetInteger(0,PanelBase,OBJPROP_BGCOLOR,White);
   ObjectSetInteger(0,PanelBase,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,PanelBase,OBJPROP_STATE,false);
   ObjectSetInteger(0,PanelBase,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,PanelBase,OBJPROP_FONTSIZE,8);
   ObjectSet(PanelBase,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,PanelBase,OBJPROP_COLOR,clrBlack);
      
   DrawEdit(PanelLabel,
            Xoff+2,
            Yoff+2,
            PanelLabX,
            PanelLabY,
            true,
            10,
            PanelToolTip,
            ALIGN_CENTER,
            "Consolas",
            PanelText,
            false,
            clrNavy,
            clrKhaki,
            clrBlack);
   
   string EnableDisabledText="";
   color EnableDisabledColor=clrNavy;
   color EnableDisabledBack=clrKhaki;
   if(EnableMonitoring){
      EnableDisabledText="MONITORING ENABLED";
      EnableDisabledColor=clrWhite;
      EnableDisabledBack=clrDarkGreen;
   }
   else{
      EnableDisabledText="MONITORING DISABLED";
      EnableDisabledColor=clrWhite;
      EnableDisabledBack=clrDarkRed;
   }
   
   DrawEdit(PanelEnableDisable,
            Xoff+2,
            Yoff+(PanelMovY+1)*Rows+2,
            PanelLabX,
            PanelLabY,
            true,
            8,
            "Click to Enable or Disable the Monitoring Feature",
            ALIGN_CENTER,
            "Consolas",
            EnableDisabledText,
            false,
            EnableDisabledColor,
            EnableDisabledBack,
            clrBlack);

   Rows++;

   
   ObjectSetInteger(0,PanelBase,OBJPROP_XSIZE,PanelRecX);
   ObjectSetInteger(0,PanelBase,OBJPROP_YSIZE,(PanelMovY+1)*Rows+3);
}


void CleanPanel(){
   int Window=0;
   for(int i=ObjectsTotal(ChartID(),Window,-1)-1;i>=0;i--){
      if(StringFind(ObjectName(i),IndicatorName+"-P-",0)>=0){
         ObjectDelete(ObjectName(i));
      }
   }
}


void ChangeMonitoringEnabled(){
   if(EnableMonitoring==false){
      if(IsTradeAllowed()) EnableMonitoring=true;
      else{
         MessageBox("You need to first enable Live Trading in your Metatrader options","WARNING",MB_OK);
      }
   }
   else EnableMonitoring=false;
   DrawPanel();
}